####################################################
#modules
import json
import numpy as np
import pandas as pd
from json import dumps
from mtbpy import mtbpy
from requests import get
import plotly.express as px
import matplotlib.pyplot as plt
import plotly.graph_objects as go
from urllib.request import urlopen
###############################
# Adding notes to Minitab Output pane

mtbpy.mtb_instance().set_note("- This project imports Covid data from .gov website through integration with Python. \n - Users can then use various functions within Minitab to explore the data." )

####################################################
# Specify URL, filters and headers

ENDPOINT = "https://api.coronavirus.data.gov.uk/v1/data"

#AREA_TYPE = "region"
AREA_TYPE = "nation"

headers = mtbpy.mtb_instance().get_column("C1")

####################################################
#Function

def CoviData(area_name,headers):
    AREA_NAME = str(area_name)

    filters = [
        f"areaType={ AREA_TYPE }",
        f"areaName={ AREA_NAME }"
    ]

    new_headers = []
    for i in headers:
        new_headers.append([i,i])
    structure = dict(new_headers)

    api_params = {
        "filters": str.join(";", filters),
        "structure": dumps(structure, separators=(",", ":"))
    }


    response = get(ENDPOINT, params=api_params, timeout=10)

    if response.status_code >= 400:
        raise RuntimeError(f'Request failed: { response.text }')

    data = response.json()
    data2 = data['data']
    data3 = pd.DataFrame.from_dict(data2)
    
    return data3

regions = ["England", "Scotland", "Wales", "Northern Ireland"]
dff = pd.DataFrame(columns=headers)

if not headers:
    mtbpy.mtb_instance().add_message("Please Enter Metrics of interest in Column C1.")
else:
    for i in regions:
        df = CoviData(i,headers)
        df['region'] =i
        dff = dff.append(df)

dff.to_csv("CovidDataByRegion.csv", index=False) 
mtbpy.mtb_instance().add_message("Data successfully sent to a Minitab Worksheet")
###############################################
#Heatmap on virus
dff_n = dff.groupby(['region']).agg({'newCasesByPublishDate':np.sum})
dff_n = dff_n.reset_index()
dff_n = dff_n.rename(columns={"newCasesByPublishDate": "positiveCases"})
df2 = dff_n.copy()

with urlopen('https://opendata.arcgis.com/datasets/ee3bb6a5a73540beab9837154fd9aa9a_0.geojson') as response:
    uk_countries=json.load(response)
    

area={}
for feature in uk_countries['features']:
    feature['id']=feature['properties']['objectid']
    area[feature['properties']['ctry19nm']]=feature['id']

dff_n['id']=dff_n['region'].apply(lambda x: area[x])
fig = px.choropleth_mapbox(dff_n, locations="id", featureidkey='properties.objectid', geojson=uk_countries, color='positiveCases', hover_name='region', mapbox_style='white-bg', zoom=4, center = {"lat": 55, "lon": 0})
fig.write_image("heatmap.png")
mtbpy.mtb_instance().add_image("heatmap.png")# Sends png file to Minitab output.
fig.show()#sends image to browser

##############################################
#Sunburst on vaccinations
dff_vac = dff[['date', 'cumVaccinationFirstDoseUptakeByPublishDatePercentage', 'cumVaccinationSecondDoseUptakeByPublishDatePercentage','region']]
dff_vac.columns = ["date", "firstDose", "secondDose","region"]
dff = dff_vac.groupby(['region']).agg({'firstDose':np.max,'secondDose':np.max})
dff = dff.reset_index()

VALUE = [0]
VALUE.append(dff.loc[dff['region'] == 'Scotland', 'firstDose'].iloc[0])
VALUE.append(dff.loc[dff['region'] == 'England', 'firstDose'].iloc[0])
VALUE.append(dff.loc[dff['region'] == 'Wales', 'firstDose'].iloc[0])
VALUE.append(dff.loc[dff['region'] == 'England', 'secondDose'].iloc[0])
VALUE.append(dff.loc[dff['region'] == 'Scotland', 'secondDose'].iloc[0])
VALUE.append(dff.loc[dff['region'] == 'Northern Ireland', 'firstDose'].iloc[0])
VALUE.append(dff.loc[dff['region'] == 'Northern Ireland', 'secondDose'].iloc[0])
VALUE.append(dff.loc[dff['region'] == 'Wales', 'secondDose'].iloc[0])

fig =go.Figure(go.Sunburst(
    labels=["Covid19", "Scotland 1st dose", "England 1st dose", "Wales 1st dose", "2nd dose", "Scotland 2nd dose", "NI 1st dose", "NI 2nd dose", "W 2nd dose"],
    parents=["", "Covid19", "Covid19", "Covid19", "England 1st dose", "Scotland 1st dose", "Covid19", "NI 1st dose", "Wales 1st dose" ],
    values=VALUE,
))
fig.update_layout(margin = dict(t=0, l=0, r=0, b=0))
fig.write_image("sunburst.png")
mtbpy.mtb_instance().add_image("sunburst.png")# Sends png file to Minitab output.
fig.show()#sends image to browser
